@@ -16,14 +16,18 @@ from PIL import Image
 
 from .custom_logger import logger
 from .name_fallback import extract_patient_info_from_text
-from .ocr import tesseract_full_image_ocr  # , trocr_full_image_ocr_on_boxes # Import OCR fallback
+from .ocr import \
+    tesseract_full_image_ocr  # , trocr_full_image_ocr_on_boxes # Import OCR fallback
 from .ocr_ensemble import ensemble_ocr  # Import the new ensemble OCR
-from .ocr_preprocessing import optimize_image_for_medical_text, preprocess_image
+from .ocr_preprocessing import (optimize_image_for_medical_text,
+                                preprocess_image)
 from .ollama_llm_meta_extraction_optimized import OllamaOptimizedExtractor
 from .pdf_operations import convert_pdf_to_images
-from .sensitive_region_cropper import SensitiveRegionCropper  # Import the new cropper
+from .sensitive_region_cropper import \
+    SensitiveRegionCropper  # Import the new cropper
 from .settings import DEFAULT_SETTINGS
-from .spacy_extractor import EndoscopeDataExtractor, ExaminationDataExtractor, ExaminerDataExtractor, PatientDataExtractor
+from .spacy_extractor import (EndoscopeDataExtractor, ExaminationDataExtractor,
+                              ExaminerDataExtractor, PatientDataExtractor)
 from .text_anonymizer import anonymize_text
 from .utils.ollama import ensure_ollama
 
@@ -379,12 +383,175 @@ class ReportReader:
 
         return anonymized_text
 
-    def process_report(self, pdf_path=None, image_path=None, use_ensemble=False, verbose=True, use_llm_extractor="deepseek", text=None):
+    def create_anonymized_pdf(self, pdf_path: str, output_path: Optional[str] = None, report_meta: Optional[Dict[str, Any]] = None) -> Optional[str]:
+        """
+        Create an anonymized PDF with sensitive regions blackened out.
+        
+        Uses EAST text detection for fast region identification and TesseOCR for 
+        accurate text recognition and sensitivity classification.
+
+        Args:
+            pdf_path: Path to the original PDF
+            output_path: Path for the anonymized PDF (optional, auto-generated if not provided)
+            report_meta: Metadata from extraction (optional, will be extracted if not provided)
+
+        Returns:
+            Path to the anonymized PDF, or None if creation failed
+        """
+        try:
+            if not output_path:
+                output_path = str(Path(pdf_path).with_stem(Path(pdf_path).stem + "_anonymized"))
+
+            # Ensure output directory exists
+            Path(output_path).parent.mkdir(parents=True, exist_ok=True)
+
+            logger.info(f"Creating anonymized PDF with EAST + TesseOCR: {output_path}")
+
+            # Import required modules
+            import fitz  # PyMuPDF
+
+            from .east_text_detection import east_text_detection
+            from .ocr_tesserocr import tesseract_on_boxes_fast
+            from .pdf_operations import convert_pdf_to_images
+
+            # Open the original PDF
+            doc = fitz.open(pdf_path)
+
+            # Convert PDF to images for analysis
+            images = convert_pdf_to_images(pdf_path)
+
+            # Process each page
+            for page_num, image in enumerate(images):
+                page = doc[page_num]
+
+                # Save temporary image for EAST detection
+                import tempfile
+                with tempfile.NamedTemporaryFile(suffix=".png", delete=False) as tmp_file:
+                    temp_image_path = tmp_file.name
+                    image.save(temp_image_path)
+
+                try:
+                    # Step 1: Use EAST for fast text region detection
+                    logger.debug(f"Running EAST text detection on page {page_num + 1}")
+                    text_boxes, confidences_json = east_text_detection(
+                        temp_image_path,
+                        min_confidence=0.5,
+                        width=640,  # Higher resolution for better detection
+                        height=640
+                    )
+
+                    if not text_boxes:
+                        logger.info(f"No text regions detected on page {page_num + 1}")
+                        continue
+
+                    logger.info(f"EAST detected {len(text_boxes)} text regions on page {page_num + 1}")
+
+                    # Step 2: Use TesseOCR for fast and accurate text extraction
+                    logger.debug(f"Running TesseOCR on {len(text_boxes)} boxes")
+                    text_with_boxes, ocr_confidences = tesseract_on_boxes_fast(
+                        image,  # Use PIL image directly
+                        text_boxes,
+                        language="deu+eng"
+                    )
+
+                    # Step 3: Detect sensitive regions based on OCR results
+                    sensitive_regions = self.sensitive_cropper.detect_sensitive_regions(
+                        image, 
+                        text_with_boxes
+                    )
+
+                    if sensitive_regions:
+                        logger.info(f"Blackening {len(sensitive_regions)} sensitive regions on page {page_num + 1}")
+
+                        # Convert pixel coordinates to PDF coordinates
+                        page_rect = page.rect
+                        page_height = page_rect.height
+                        page_width = page_rect.width
+
+                        img_width, img_height = image.size
+
+                        # Scaling factors
+                        scale_x = page_width / img_width
+                        scale_y = page_height / img_height
+
+                        for x1, y1, x2, y2 in sensitive_regions:
+                            # Convert image coordinates to PDF coordinates
+                            pdf_x1 = x1 * scale_x
+                            pdf_y1 = page_height - (y2 * scale_y)  # Invert Y-axis
+                            pdf_x2 = x2 * scale_x
+                            pdf_y2 = page_height - (y1 * scale_y)  # Invert Y-axis
+
+                            # Create black rectangle with slight padding for better coverage
+                            padding = 2  # PDF points
+                            rect = fitz.Rect(
+                                pdf_x1 - padding, 
+                                pdf_y1 - padding, 
+                                pdf_x2 + padding, 
+                                pdf_y2 + padding
+                            )
+
+                            # Add black rectangle to cover sensitive area
+                            page.draw_rect(rect, color=(0, 0, 0), fill=(0, 0, 0))
+
+                            logger.debug(f"Blackened: ({pdf_x1:.1f}, {pdf_y1:.1f}, {pdf_x2:.1f}, {pdf_y2:.1f})")
+                    else:
+                        logger.info(f"No sensitive regions detected on page {page_num + 1}")
+
+                finally:
+                    # Clean up temporary file
+                    import os
+                    try:
+                        os.unlink(temp_image_path)
+                    except Exception:
+                        pass
+
+            # Save the anonymized PDF
+            doc.save(output_path)
+            doc.close()
+
+            logger.info(f"Anonymized PDF saved: {output_path}")
+            return output_path
+
+        except ImportError as e:
+            logger.error(f"Required module not installed. Cannot create anonymized PDF: {e}")
+            return None
+        except Exception as e:
+            logger.error(f"Error creating anonymized PDF: {e}", exc_info=True)
+            return None
+
+    def process_report(
+        self,
+        pdf_path=None,
+        image_path=None,
+        use_ensemble=False,
+        verbose=True,
+        use_llm_extractor="deepseek",
+        text=None,
+        create_anonymized_pdf=False,
+        anonymized_pdf_output_path=None,
+    ):
         """
         Process a report by extracting text, metadata, and creating an anonymized version.
         If the normal pdfplumber extraction fails (or returns very little text), fallback to OCR.
         Optionally, use an ensemble OCR method to improve output quality.
         Optionally, specify an LLM extractor ('deepseek', 'medllama', 'llama3') to use INSTEAD of SpaCy/regex.
+
+        Args:
+            pdf_path: Path to the PDF file
+            image_path: Path to an image file (alternative to PDF)
+            use_ensemble: Whether to use ensemble OCR for better quality
+            verbose: Enable verbose logging
+            use_llm_extractor: LLM model to use ('deepseek', 'medllama', 'llama3', or None for SpaCy/regex)
+            text: Pre-extracted text (optional, will extract from PDF/image if not provided)
+            create_anonymized_pdf: If True, creates a PDF with sensitive regions blackened out
+            anonymized_pdf_output_path: Custom path for the anonymized PDF (auto-generated if not provided)
+
+        Returns:
+            Tuple: (original_text, anonymized_text, report_meta, anonymized_pdf_path)
+                - original_text: Extracted text from the document
+                - anonymized_text: Text with sensitive information replaced
+                - report_meta: Dictionary with extracted metadata (names, dates, etc.)
+                - anonymized_pdf_path: Path to the blackened PDF (None if not created)
         """
         if text is None:
             if not pdf_path and not image_path:
@@ -497,7 +664,8 @@ class ReportReader:
                     logger.info("Applying LLM correction to OCR text via Ollama")
                     try:
                         # Use the existing ollama_service for correction
-                        from .ollama_service import ollama_service  # Import locally if needed
+                        from .ollama_service import \
+                            ollama_service  # Import locally if needed
 
                         # Ensure the desired correction model is set up if different from extraction
                         # ollama_service.setup_ollama("deepseek-r1:1.5b") # Or another model
@@ -556,6 +724,21 @@ class ReportReader:
         # --- Anonymization ---
         anonymized_text = self.anonymize_report(text=text, report_meta=report_meta)
 
+        # --- Create Anonymized PDF (if requested) ---
+        anonymized_pdf_path = None
+        if create_anonymized_pdf and pdf_path:
+            try:
+                logger.info("Creating anonymized PDF with blackened sensitive regions...")
+                anonymized_pdf_path = self.create_anonymized_pdf(pdf_path=pdf_path, output_path=anonymized_pdf_output_path, report_meta=report_meta)
+                if anonymized_pdf_path:
+                    report_meta["anonymized_pdf_path"] = anonymized_pdf_path
+                    logger.info(f"Anonymized PDF created: {anonymized_pdf_path}")
+                else:
+                    logger.warning("Failed to create anonymized PDF")
+            except Exception as e:
+                logger.error(f"Error creating anonymized PDF: {e}")
+                report_meta["anonymized_pdf_error"] = str(e)
+
         # Log final outcome
         if ocr_applied:
             logger.info(f"Processed (OCR applied): {pdf_path}. Meta keys found: {list(report_meta.keys())}")
@@ -564,7 +747,7 @@ class ReportReader:
         logger.debug(f"Final Report Meta: {report_meta}")
         # logger.debug(f"Anonymized text preview: {anonymized_text[:200]}...") # Be careful logging anonymized text
 
-        return text, anonymized_text, report_meta
+        return text, anonymized_text, report_meta, anonymized_pdf_path
 
     def pdf_hash(self, pdf_binary):
         """
@@ -715,3 +898,13 @@ class ReportReader:
             logger.info(f"Visualisierung erstellt: {vis_filename}")
 
         return visualization_files
+
+            vis_filename = f"{pdf_name}_page_{page_num + 1}_analysis.png"
+            vis_path = output_dir / vis_filename
+
+            self.sensitive_cropper.visualize_sensitive_regions(image, word_boxes, str(vis_path))
+
+            visualization_files.append(str(vis_path))
+            logger.info(f"Visualisierung erstellt: {vis_filename}")
+
+        return visualization_files
